# Method define in Activesupport
class Array
  def sum
    return self.inject{|sum,x| sum + x }
  end
end

# dummy User class for testing.
# in real app it should be subclassed from ActiveRecord::Base 
# and have at least these attributes:
#   - role (Integer)
#   - identity_url (String)
class User
  attr_accessor :new_record, :role, :identity_url
  alias_method :new_record?, :new_record
  
  def initialize(*args)
    super(*args)
    @new_record = true
  end
  
  # stub methods and attributes
  def self.scope(*args); end
  
  include DisiidUser
end

class FakeResponse
  attr_accessor :body
  def initialize(body); @body = body; end
end

# Stub ActiveRecord so that it won't do real HTTP requests
module ActiveResource
  class Connection
    # taps into ActiveRecord to read data from stub files
    # instead of doing a real HTTP request
#    def get(path, *args)
#      read_from_file(path)
#    rescue; raise ActiveResource::ResourceNotFound; end
    
    # stubs PUT request so to not do it for real
#    def put(path, payload, headers)
#      read_from_file(path)
#    rescue; raise ActiveResource::ResourceNotFound; end
    
    private 
    
    def read_from_file(fake_url_path)
      if matched = /users\.(xml|json)/.match(fake_url_path)
        filename = "#{File.dirname(__FILE__)}/users.#{matched[1]}"
      else
        # /collection_name/uuid?query
        record = /\/[^\/]+\/([^\/\?]+)/.match(fake_url_path)[1]
        # record string includes .xml or .json too
        filename = "#{File.dirname(__FILE__)}/#{record}".sub(/\?.*$/, '')
      end
      # format is either ActiveResource::Formats::XmlFormat or JsonFormat
      # 3.0.x defaults to XML, 3.1.x defaults to JSON
      data = open(filename).read
      /3\.1/.match(ActiveResource::VERSION::STRING) ? FakeResponse.new(data) : format.decode(data)
    end
  end
end

# stub implementation for Rails.cache
# it will never really cache results 
# so it is safe to Rails.cache.fetch(...) { ... }
class StubCache
  def self.fetch(*args); yield; end
end

module Rails
  def self.cache; StubCache; end
end

